package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperAnalyzeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeQuestionMapper;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.sexam.vo.paper.EPaperQPointVo;
import com.xhkjedu.sexam.vo.paper.EPaperQsVo;
import com.xhkjedu.sexam.vo.paper.EPaperQtypeQuestionVo;
import com.xhkjedu.sexam.vo.paper.EPaperQtypeVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 题库题型
 * @Author WN
 * Date 2022/7/19 9:07
 **/
@Service
public class EPaperQtypeService {
    @Resource
    private EPaperQtypeMapper ePaperQtypeMapper;
    @Resource
    private EPaperQtypeQuestionMapper ePaperQtypeQuestionMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;


    /**
     * @Description 保存题库试卷
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveQtPaper(EPaper paper) {
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题
        setEPaperBaseDetail(paper,eSubject,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);
        //保存作业分析
        savePaperAnalyze(paper);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    //设置试卷基本信息
    private void setEPaperBaseDetail(EPaper paper,ESubject eSubject,List<EPaperQtype> qtypelist,List<EPaperQtypeQuestion> queslist){
        qtypelist.forEach(q ->{
            queslist.addAll(q.getQuestions());
        });
        paper.setPtype(1);//试卷类型1题库2答题卡3仅答题卡
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        paper.setEsid(eSubject.getEsid());
        int timestamp = N_Utils.getSecondTimestamp();
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperQtypeQuestion:: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        paper.setAnswered(1);
    }

    //保存试卷题型试题信息
    private void savePaperQtype(EPaper epaper,List<EPaperQtype> qtypelist){
        List<EPaperQtypeQuestion> qtqs = new ArrayList<>();
        for(EPaperQtype qtype : qtypelist){
            List<EPaperQtypeQuestion> qlst = qtype.getQuestions();
            qtype.setEpid(epaper.getEpid());
            qtype.setCreateid(epaper.getHandleid());
            qtype.setCreatetime(epaper.getHandletime());
            qtype.setEptnum(qlst.size());
            Double ptscore = qlst.stream().collect(Collectors.summingDouble(EPaperQtypeQuestion::getScore));
            qtype.setEptscore(ptscore);
        }
        ePaperQtypeMapper.insertList(qtypelist);//保存题型
        for(EPaperQtype qtype : qtypelist){
            for(EPaperQtypeQuestion q : qtype.getQuestions()){
                q.setEpid(epaper.getEpid());
                q.setEptid(qtype.getId());
                qtqs.add(q);
            }
        }
        ePaperQtypeQuestionMapper.insertList(qtqs);
    }

    //保存试卷分析List<EPaperQtype> qtypelist,List<EPaperQtypeQuestion> queslist
    public void savePaperAnalyze(EPaper ePaper){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        List<EPaperQtypeVo> qtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(ePaper.getEpid());//试卷题型试题信息
        List<EPaperQtypeQuestionVo> queslist = new ArrayList<>();//试卷中所有试题
        List<EPaperQPointVo> quespoints = new ArrayList<>();//试卷中所有试题包含子题已经题号和对应知识点

        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        //处理出试卷中所有试题、知识点
        for(EPaperQtypeVo qt : qtypelist){
            for(EPaperQtypeQuestionVo q : qt.getQuestions()){
                queslist.add(q);
                if(q.getQlevel() == 0 && q.getHasson() == 1){
                    //说明试题有子题
                    /*for(EPaperQtypeQuestionVo sq : q.getSonquestions()){
                        List<EPaperQPointVo> pointslist = sq.getPoints();
                        if(pointslist != null && pointslist.size()>0){
                            for(EPaperQPointVo point : pointslist){
                                EPaperQPointVo p = new EPaperQPointVo();
                                p.setPointid(point.getPointid());
                                p.setPointname(point.getPointname());
                                p.setQuestionid(sq.getQuestionid());
                                p.setOrder(q.getOrder());
                                p.setScore(sq.getScore());
                                quespoints.add(p);
                            }
                        }
                    }*/
                }else{
                    List<EPaperQPointVo> pointslist = q.getPoints();
                    if(pointslist != null && pointslist.size()>0){
                        for(EPaperQPointVo point : pointslist){
                            EPaperQPointVo p = new EPaperQPointVo();
                            p.setPointid(point.getPointid());
                            p.setPointname(point.getPointname());
                            p.setQuestionid(q.getQuestionid());
                            p.setOrder(q.getOrder());
                            p.setScore(q.getScore());
                            quespoints.add(p);
                        }
                    }
                }
            }
        }

        //总体分布
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        for(EPaperQtypeQuestionVo q : queslist){
            if(!N_Utils.isObjectiveQuestion(q.getCtype())){
                znum ++;
                zscore = zscore + q.getScore();
            }else{
                knum ++;
                kscore = kscore + q.getScore();
            }
        }
        double znumrate = N_Utils.getIntegerDivideAndMulitiply(znum, ePaper.getPnum());
        double knumrate = 100-znumrate;
        double zscorerate = N_Utils.getDoubleDivideAndMulitiply(zscore, ePaper.getPscore());
        double kscorerate = 100 - zscorerate;

        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("主观题",zscore,znum,zscorerate,znumrate));
        ranglist.add(new EPaperQsVo("客观题",kscore,knum,kscorerate,knumrate));

        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //题型分布
        queslist.stream().sorted(Comparator.comparing(EPaperQtypeQuestionVo::getOrder)).collect(Collectors.toList());
        Map<String,List<EPaperQtypeQuestionVo>> qtypemap =  queslist.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getQtypeid, Collectors.toList()));
        List<EPaperQsVo> typelist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperQtypeQuestionVo>> entry : qtypemap.entrySet()){
            List<EPaperQtypeQuestionVo> aq = entry.getValue();
            double tscore = aq.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
            double storerate = N_Utils.getDoubleDivideAndMulitiply(tscore, ePaper.getPscore());
            EPaperQsVo tvo = new EPaperQsVo();
            tvo.setQtname(aq.get(0).getQtypename());
            tvo.setNum(aq.size());
            tvo.setScore(tscore);
            tvo.setSrate(storerate);
            typelist.add(tvo);
        }
        //难易度
        Map<Integer,List<EPaperQtypeQuestionVo>> complxmap =  queslist.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getComplexity, Collectors.toList()));
        List<Map> clist = new ArrayList<>();
        for(Map.Entry<Integer, List<EPaperQtypeQuestionVo>> entry : complxmap.entrySet()){
            Map cmap = new HashMap();
            cmap.put("level",entry.getKey());
            List<EPaperQtypeQuestionVo> aq = entry.getValue();
            double cscore = aq.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            cmap.put("score",cscore);
            cmap.put("srate",scorerate);
            clist.add(cmap);
        }

        //知识点
        Map<String,List<EPaperQPointVo>> pointmap =  quespoints.stream().collect(Collectors.groupingBy(EPaperQPointVo:: getPointid, Collectors.toList()));
        List<Map> pointlist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperQPointVo>> entry : pointmap.entrySet()){
            Map pm = new HashMap();
            List<EPaperQPointVo> pvo = entry.getValue();
            double cscore = pvo.stream().collect(Collectors.summingDouble(EPaperQPointVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            List<Integer> thlist = pvo.stream().map(EPaperQPointVo :: getOrder).collect(Collectors.toList());
            pm.put("pointname",pvo.get(0).getPointname());
            pm.put("score",cscore);
            pm.put("srate",scorerate);
            pm.put("order",thlist);
            pointlist.add(pm);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));//总体分析
        epa.setQnumjson(JSON.toJSONString(typelist));//试题栏题型分析
        epa.setComplexityjson(JSON.toJSONString(clist));//难易度分析
        epa.setPointjson(JSON.toJSONString(pointlist));//知识点分析
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);

    }

    /**
     * @Description 修改试卷试题
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/20 8:43
     **/
    @Transactional(rollbackFor = Exception.class)
    public void updateQtPaper(EPaper paper) {
        //删除原试卷
        ePaperMapper.deletePaperByEpid(paper.getEpid());
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题
        setEPaperBaseDetail(paper,eSubject,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);
        //保存作业分析
        savePaperAnalyze(paper);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    /**
     * @Description  题库试卷详情
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    public Map getPaperQtDetail(Integer epid) {
        Map map = ePaperMapper.getExamPaperDetailByEpid(epid);//试卷信息
        List<EPaperQtypeVo> fjtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(epid);//试卷题型试题信息
        map.put("qtypes",fjtypelist);
        return map;
    }
}
